#- coding: utf-8
from . import errors
from . import registro
from . import tipos
from . import bancos
from . import eventos
from . import lotes
from .. import datas
from .. import calcDigito
from ..formatastring import formataString
from ..numero import getNumToStr


OCORRENCIAS_PGTO = {
    '00': 'Pagamento efetuado',
    'AE': 'Data pagamento alterada',
    'AG': 'No lote invalido',
    'AH': 'No sequencial registro no lote invalido',
    'AJ': 'Tipo movimento invalido',
    'AL': 'Codigo banco favorecido invalido',
    'AM': 'Agencia favorecido invalida',
    'AO': 'Favorecido invalido',
    'AP': 'Data pagamento invalida',
    'AQ': 'Tem mais de 999999 registros',
    'BC': 'Nosso numero invalido',
    'BD': 'Pag.agendado',
    'CD': 'CNPJ/CPF divergente',
    'CE': 'Pagamento cancelado',
    'CF': 'Valor documento invalido',
    'CH': 'Valor desconto invalido',
    'CK': 'Tipo de inscricao invalida',
    'CN': 'Conta nao cadastrada',
    'HA': 'Erro no header do arquivo',
    'HM': 'Erro no header do lote',
    'IB': 'Valor documento invalido',
    'IC': 'Valor abatimento invalido',
    'ID': 'Valor desconto invalido',
    'IH': 'Valor acrescimo invalido',
    'II': 'Data vencimento invalida',
    'IM': 'Tipo X Forma nao compativel',
    'IN': 'Banco/Agencia nao cadastrados',
    'IP': 'Dac codigo barras invalido',
    'IR': 'Pagamento alterado',
    'IS': 'Concessionaria nao conveniada',
    'NR': 'Operacao nao realizada',
    'RJ': 'Registro rejeitado',
    'SS': 'Sem Saldo/Sem limite',
    'TA': 'Totais do lote com diferenca'
}

def cnab_get_conta(nroBacen, conta):
    __conta = conta
    if nroBacen == '001':
        __conta = formataString(str(conta), 8, '0', 'E', 'S', 'S')
    if nroBacen in ('237', '756'):
        __conta = formataString(str(conta), 7, '0', 'E', 'S', 'S')
    if nroBacen == '341':
        __conta = formataString(str(conta), 5, '0', 'E', 'S', 'S')
    return __conta

def cnab_get_cb_campo_livre(nroBacen, agencia, conta, carteira, nossoNumeroSeq, convenio, parcelaDoTitulo, codigoSacado,
                            documento):
    """
    Calcula o campo chamado campo livre do codigo de barras bancario
    :param nroBacen:
    :param agencia:
    :param conta:
    :param carteira:
    :param nossoNumeroSeq:
    :param convenio:
    :param parcelaDoTitulo:
    :return: string com o campo livre tamanho de 25 bytes
    """
    def __get_bco_001(carteira, agencia, conta, convenio, nossoNumeroSeq):
        __carteira = formataString(str(carteira), 2, '0', 'E', 'S', 'S')
        __agencia = agencia
        __convenio = convenio
        __nossoNumeroSeq = nossoNumeroSeq
        _contaDV = calcDigito.baseModulo(conta, '98765432', 11, subResto='N')
        if _contaDV == '10':
            _contaDV = 'X'
        _tamConvenio = len(__convenio)
        if _tamConvenio < 5:
            _tamConvenio = 4
            __convenio = formataString(__convenio, 4, '0', 'D')
            __nossoNumeroSeq = formataString(__nossoNumeroSeq, 7, '0', 'D', 'S', 'S')
            __nossoNumeroSeq = __convenio + __nossoNumeroSeq
            __nossoNumeroCompletoEditado = __nossoNumeroSeq
            __campoLivre = __nossoNumeroSeq + __agencia + conta + __carteira
        if _tamConvenio > 4 and _tamConvenio < 7:
            _tamConvenio = 6
            __convenio = formataString(__convenio, 6, '0', 'D')
            __nossoNumeroSeq = formataString(__nossoNumeroSeq, 5, '0', 'D', 'S', 'S')
            __nossoNumeroCompleto = __convenio + __nossoNumeroSeq
            __nossoNumeroCompletoEditado = __nossoNumeroCompleto
            __campoLivre = __nossoNumeroCompleto + __agencia + conta + __carteira
        if _tamConvenio > 6:
            __nossoNumeroSeq = formataString(__nossoNumeroSeq, 10, '0', 'E', 'S', 'S')
            __nossoNumeroCompleto = __convenio + __nossoNumeroSeq
            __nossoNumeroCompletoEditado = __nossoNumeroCompleto
            __campoLivre = '000000' + __nossoNumeroCompleto + __carteira
        if not __convenio:
            __nossoNumeroCompleto = formataString(__nossoNumeroSeq, 17, '0', 'E', 'S', 'S')
            __nossoNumeroCompletoEditado = __nossoNumeroCompleto
            __campoLivre = '000000' + __nossoNumeroCompleto + __carteira
        _dv = calcDigito.baseModulo(__nossoNumeroCompleto, '23456789', 11, 'N')
        if _dv == '10':
            _dv = 'X'
        _agenciaDV = calcDigito.baseModulo(__agencia, '98765432', 11, subResto='N')
        if _agenciaDV == '10':
            _agenciaDV = 'X'
        __agencia = __agencia + '-' + _agenciaDV
        __conta = conta + '-' + _contaDV
        __nossoNumeroCompleto = __nossoNumeroCompleto+ _dv
        __nossoNumeroCompletoEditado = __nossoNumeroCompleto
        return __campoLivre

    def __get_bco_104(carteira, convenio, nossoNumeroSeq):
        __carteira = formataString(str(carteira), 2, '0', 'E', 'S', 'S')
        __convenio = formataString(str(convenio), 7, '0', 'E', 'S', 'S')
        __nossoNumeroSeq = nossoNumeroSeq
        __nossoNumeroCompleto = formataString(nossoNumeroSeq, 15, '0', 'E', 'S', 'S')
        __nossoNumeroCompleto = '24' + __nossoNumeroCompleto
        _dv = calcDigito.baseModulo(__nossoNumeroCompleto, '98765432', 11)
        if _dv in ('10', '11'):
            _dv = '0'
        __nossoNumeroCompleto = __nossoNumeroCompleto + _dv
        __nossoNumeroCompletoEditado = __nossoNumeroCompleto[:2] + '/' + \
                                       __nossoNumeroCompleto[2:-1] + '-' + \
                                       __nossoNumeroCompleto[-1:]
        __campoLivre = __convenio + __nossoNumeroCompleto[2:5] + \
                            __nossoNumeroCompleto[:1] + \
                            __nossoNumeroCompleto[5:8] + __nossoNumeroCompleto[1:2] + \
                            __nossoNumeroCompleto[8:17]
        _dv = calcDigito.baseModulo(__campoLivre, '98765432', 11)
        if _dv in ('10', '11'):
            _dv = '0'
        __campoLivre = __campoLivre + _dv
        return __campoLivre

    def __get_bco_237(carteira, agencia, conta, nossoNumeroSeq):
        __carteira = formataString(str(carteira), 2, '0', 'E', 'S', 'S')
        __agencia = agencia
        __nossoNumeroCompleto = formataString(nossoNumeroSeq, 11, '0', 'D', 'S', 'S')
        __campoLivre = __agencia + __carteira + __nossoNumeroCompleto + conta + '0'
        _dv = calcDigito.baseModulo(__carteira + __nossoNumeroCompleto, '765432', 11)
        if _dv == '10':
            _dv = 'P'
        if _dv == '11':
            _dv = '0'
        __nossoNumeroCompleto = __nossoNumeroCompleto + _dv
        __nossoNumeroCompletoEditado = __nossoNumeroCompleto
        return __campoLivre

    def __get_bco_341(carteira, agencia, conta, nossoNumeroSeq):
        __carteira = formataString(str(carteira), 3, '0', 'E', 'S', 'S')
        __nossoNumeroSeq = formataString(nossoNumeroSeq, 8, '0', 'E', 'S', 'S')
        __nossoNumeroCompleto = __carteira + __nossoNumeroSeq
        _dv = calcDigito.baseModulo(__nossoNumeroCompleto, '12', 10, somaDigitos='S')
        if _dv == '10':
            _dv = '0'
        __nossoNumeroCompleto = __nossoNumeroCompleto + _dv
        __nossoNumeroCompletoEditado = (__nossoNumeroCompleto[:3] + '/'
                                           + __nossoNumeroCompleto[3:-1] + '-'
                                           + __nossoNumeroCompleto[-1:]
                                           )
        _ac_dv = calcDigito.baseModulo(agencia + conta, '12', 10, somaDigitos='S')
        if _ac_dv == '10':
            _ac_dv = '0'
        __campoLivre = __carteira + __nossoNumeroSeq + _dv + agencia + conta + _ac_dv + '000'
        return __campoLivre

    def __get_bco_756(carteira, agencia, convenio, nossoNumeroSeq, parcela):
        __carteira = formataString(str(carteira), 1, '0', 'E', 'S', 'S')
        __convenio = convenio
        if len(__convenio) > 7:
            __convenio = __convenio[-7:]
        else:
            __convenio = str(__convenio).zfill(7)
        if len(str(parcela)) > 3:
            __parcela = parcela[-3:]
        else:
            __parcela = str(parcela).zfill(3)
        # O numero do cliente eh o dado informado no campo convenio do cadastro de local e
        # natureza
        __clienteNossoNumero = __convenio.zfill(10)
        __agenciaNossoNumero = str(agencia).zfill(4)
        __nossoNumeroSeq = str(nossoNumeroSeq).zfill(7)
        __nossoNumeroCompleto = __agenciaNossoNumero + __clienteNossoNumero + __nossoNumeroSeq
        __nossoNumeroCompleto = formataString(__nossoNumeroCompleto, 21, '0', 'D', 'S', 'S')
        _dv = calcDigito.baseModulo(__nossoNumeroCompleto, '1973', 11, 'S', 'N')
        if _dv == '10':
            _dv = '0'
        if _dv == '11':
            _dv = '0'
        __nossoNumeroCompleto = __nossoNumeroSeq + _dv
        __nossoNumeroCompletoEditado = __nossoNumeroCompleto
        __campoLivre = __carteira + agencia + '02' + __convenio + __nossoNumeroCompleto + __parcela
        return __campoLivre

    def __get_bco_sb(carteira, nossoNumeroSeq, sacadoCodigo, documento):
        __carteira = formataString(str(carteira), 2, '0', 'E', 'S', 'S')
        __nossoNumeroSeq = formataString(nossoNumeroSeq, 11, '0', 'E', 'S', 'S')
        __nossoNumeroCompleto = __nossoNumeroSeq
        __nossoNumeroCompletoEditado = __nossoNumeroCompleto
        __campoLivre = str(sacadoCodigo) + str(documento)
        __campoLivre = formataString(__campoLivre, 25, '0', 'D', 'S', 'S')
        return __campoLivre

    conta = cnab_get_conta(nroBacen, conta)
    if nroBacen == '001':  # Banco do Brasil
        __campoLivre = __get_bco_001(carteira, agencia, conta, convenio, nossoNumeroSeq)
    elif nroBacen == '104':  # Caixa
        __campoLivre = __get_bco_104(carteira, convenio, nossoNumeroSeq)
    elif nroBacen == '237':  # Bradesco
        __campoLivre = __get_bco_237(carteira, agencia, conta, nossoNumeroSeq)
    elif nroBacen == '341':  # Itau
        __campoLivre = __get_bco_341(carteira, agencia, conta, nossoNumeroSeq)
    elif nroBacen == '756':  # Sicoob
        __campoLivre = __get_bco_756(carteira, agencia, convenio, nossoNumeroSeq, parcelaDoTitulo)
    else:
        __campoLivre = __get_bco_sb(carteira, nossoNumeroSeq, codigoSacado, documento)
    return __campoLivre


def cnab_get_codigo_barra(brw_local_natureza, brw_titulo, nossoNumeroSeq):
    """

    :param brw_local_natureza: Browser da classe local_natureza
    :param  brw_titulo: browser da classe financeiro_titulo
    :param nossoNumeroSeq: Nova sequencia de nosso numero, só a parte sequencial, se não informado, vai buscar do titulo
    :return:  3 variaveis: res(True ou false), codigo digitavel e codigo escaneavel
    """

    moeda = '9'
    nroBacen = str(brw_local_natureza.finlon_local_cobranca_id.finloc_banco_id.admbco_codigo_bc or 0)
    agencia = getNumToStr(brw_local_natureza.finlon_agencia, 4, 0, 'N')
    conta = cnab_get_conta(nroBacen, brw_local_natureza.finlon_conta)
    carteira = brw_local_natureza.finlon_carteira or '0'
    convenio = brw_local_natureza.finlon_convenio or ''

    fator = str(datas.calcDiferenca('19971007', brw_titulo.data_prorrogacao))
    parcelaDoTitulo = brw_titulo.parcela or '1'
    codigoSacado = brw_titulo.pessoa_id.codigo
    documento = brw_titulo.documento
    campo_livre = cnab_get_cb_campo_livre(nroBacen, agencia, conta, carteira, nossoNumeroSeq, convenio,
                            parcelaDoTitulo, codigoSacado, documento)

    _valor = getNumToStr(brw_titulo.vr_original, 8, 2, 'N')
    _codigoBarra = nroBacen + moeda + 'X' + fator + _valor + campo_livre
    _dv = calcDigito.baseModulo(_codigoBarra.replace('X', ''), '98765432', 11)
    if _dv in ('0', '10', '11'):
        _dv = '1'
    _codigoBarraDV = _dv
    _codigoBarra = _codigoBarra.replace('X', _codigoBarraDV)
    _campoDig1 = _codigoBarra[:4] + _codigoBarra[19:24]
    _campoDig2 = _codigoBarra[24:34]
    _campoDig3 = _codigoBarra[34:]
    _campoDig5 = _codigoBarra[5:19]
    _dv = calcDigito.baseModulo(_campoDig1, '12', 10, somaDigitos='S')
    if _dv == '10':
        _dv = '0'
    _campoDig1 = _campoDig1 + _dv
    _dv = calcDigito.baseModulo(_campoDig2, '12', 10, somaDigitos='S')
    if _dv == '10':
        _dv = '0'
    _campoDig2 = _campoDig2 + _dv
    _dv = calcDigito.baseModulo(_campoDig3, '12', 10, somaDigitos='S')
    if _dv == '10':
        _dv = '0'
    _campoDig3 = _campoDig3 + _dv
    _linhaDigitavel = _campoDig1[:5]+ _campoDig1[5:] \
                      + _campoDig2[:5] + _campoDig2[5:] \
                      + _campoDig3[:5] + _campoDig3[5:]\
                      + _codigoBarraDV \
                      + _campoDig5
    return calcDigito.codigo_barra_bancario(_linhaDigitavel)

